#!/bin/bash
SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" >/dev/null 2>&1 && pwd)"

# Load colors and the I/O functions
source ${SCRIPT_DIR}/utils/io.sh

cwd=$(pwd)

WORDPRESS_LANGUAGE=US
WORDPRESS_VERSION="5.5.3"

DEBUG_LOG="${cwd}/deploy-debug.txt"
DOCKER_REPO="infomakerscandinaviaab"
NODE_BUILD_CMD="gulp"
NODE_BUILD_STAGE_CMD="gulp --production"
NODE_BUILD_PROD_CMD="gulp --production"

# Default options
PRODUCTION=false
STAGE=false
start_local_environment=false

# Read arguments
POSITIONAL=()
while [[ $# -gt 0 ]]; do
  key="$1"
  case $key in
  --production)
    PRODUCTION=true
    shift # past argument
    ;;
  --stage)
    STAGE=true
    shift # past argument
    ;;
  --start-local-env)
    start_local_environment=true
    shift # past argument
    ;;
  *) # unknown option
    POSITIONAL+=("$1") # save it in an array for later
    shift              # past argument
    ;;
  esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters
# Done reading arguments

# Consider init for aws if flagged with "--stage" or "--production"
FOR_AWS=$(${PRODUCTION} && echo ${PRODUCTION} || echo ${STAGE})

function extractPhpVersion() {
  local __FOR_AWS=${1:-false}

  if ${__FOR_AWS}; then
    echo $(grep 'amazonaws.com/php-fpm' Dockerrun.aws.json | cut -d ':' -f3 | cut -d '"' -f1)
  else
    echo $(grep "${DOCKER_REPO}/php-fpm" docker-compose.yml | cut -d ':' -f3)
  fi
}

# Create a .env.example file
[[ ! -f .project-env ]] || [[ -f .env.example ]] || mv .project-env .env.example

# Create a .env file
[[ ! -f .env.example ]] || [[ -f .env ]] || cp .env.example .env

# Create project .gitignore
[[ ! -f .project-gitignore ]] || [[ -f .gitignore ]] || mv .project-gitignore .gitignore

if command -v docker >/dev/null 2>&1; then
  ioInfo "$(docker --version)"
else
  ioError "Docker not found. Install it and rerun this script."
  exit 1
fi

# Extract the PHP version that the project will run with
PHP_VERSION=$(extractPhpVersion ${FOR_AWS})

if [[ -z ${PHP_VERSION} ]]; then
  ioError "Could not verify projects PHP version."
  exit 1
fi

if [[ ! -f "${cwd}/composer.json" ]]; then
  ioError "Could not find composer.json."
  exit 1
fi

# Wordpress installation
WP_ARGS=(
  "--version ${WORDPRESS_VERSION}"
  "--language ${WORDPRESS_LANGUAGE}"
)

if ${FOR_AWS}; then
  WP_ARGS+=("--force")
fi

${SCRIPT_DIR}/wordpress-install.sh "${WP_ARGS[@]}"
if [[ $? -ne 0 ]]; then
  ioError "\"Wordpress install\" exited with error."
  exit 1
fi

lualibs=$(find "${cwd}/lua/includes" -maxdepth 1 -type d | wc -l)

if ${FOR_AWS} || [[ "${lualibs}" -eq 1 ]]; then
  LUAROCKS_VERSION="latest"

  ioInfo "Using \"${DOCKER_REPO}/luarocks:${LUAROCKS_VERSION}\" to install Lua libraries into lua/includes/"
  docker run --rm \
    --volume ${cwd}/rockspec:/install \
    --volume ${cwd}/lua/includes:/rocks \
    ${DOCKER_REPO}/luarocks:${LUAROCKS_VERSION}

  if [[ $? -ne 0 ]]; then
    ioError "\"Luarocks install\" exited with error."
    exit 1
  fi
fi

## Manage Composer dependencies
if ${FOR_AWS}; then
  CMDS=("install --no-dev" "dump-autoload -o")
else
  CMDS=("install")
fi

# Checking composer cache directory
[[ -d $HOME/.composer ]] || mkdir $HOME/.composer

ioInfo "Using \"${DOCKER_REPO}/php-fpm:${PHP_VERSION}\" to install composer dependencies."
for CMD in "${CMDS[@]}"; do
  SSH_DIR=$([[ -d ${cwd}/.ssh ]] && echo ${cwd}/.ssh || echo $HOME/.ssh)
  docker run --rm \
    --volume ${cwd}:/app \
    --volume ${SSH_DIR}:/root/.ssh \
    --volume $HOME/.composer:/root/.composer \
    ${DOCKER_REPO}/php-fpm:${PHP_VERSION} composer ${CMD}

  if [[ $? -ne 0 ]]; then
    ioError "\"composer ${CMD}\" exited with error."
    exit 1
  fi
done

# Install Node, gulp and run build command if gulpfile actually exists
if [[ -f ${cwd}/gulpfile.js ]]; then

  # Exit if no package.json can be found
  if [[ ! -f "${cwd}/package.json" ]]; then
    ioWarning "Can't find package.json."
    exit 0
  fi

  BUILD_CMD=${NODE_BUILD_CMD}

  if ${PRODUCTION}; then
    BUILD_CMD=${NODE_BUILD_PROD_CMD}
  elif ${STAGE}; then
    BUILD_CMD=${NODE_BUILD_STAGE_CMD}
  fi

  # Check NPM cache directory
  [[ -d $HOME/.npm ]] || mkdir $HOME/.npm

  NODE_BUILDER_VERSION="1.0.0"
  ioInfo "Using \"${DOCKER_REPO}/navigaweb-node-builder:${NODE_BUILDER_VERSION}\" to install and building application assets with command: '${BUILD_CMD}'"
  docker run --rm \
    --volume ${cwd}:/app \
    --volume $HOME/.npm:/.npm \
    ${DOCKER_REPO}/navigaweb-node-builder:${NODE_BUILDER_VERSION} ${BUILD_CMD}

  if [[ $? -ne 0 ]]; then
    ioError "Something went wrong when running the gulp task!"
    exit 1
  fi
fi

if [[ ${start_local_environment} == true ]]; then
  make pull
  make start
fi
