const gulp = require('gulp');

// CSS related plugins.
const sass = require('gulp-sass'), // Gulp pluign for Sass compilation
    sassLint = require('gulp-sass-lint'), // Linter
    minifyCss = require('gulp-clean-css'), // Minifies CSS files
    autoprefixer = require('gulp-autoprefixer'), // Adds cross-browser prefixes
    pxtorem = require('gulp-pxtorem'); // Converts pixel values to rem

// JS related plugins.
const browserify = require('browserify'),
    concat = require('gulp-concat'), // Concatenates JS files
    esLint = require('gulp-eslint'),
    uglify = require('gulp-uglify'); // Minifies JS files

// Utility related plugins.
const source = require('vinyl-source-stream'),
    buffer = require('vinyl-buffer'),
    rename = require('gulp-rename'), // Renames files E.g. style.css -> style.min.css
    sourcemaps = require('gulp-sourcemaps'), // Maps code in a compressed file (E.g. style.css) back to it’s original position in a source file
    log = require('fancy-log'),
    through2 = require('through2'), // Provides a stream to pass data through without modifying it
    argv = require('minimist')(process.argv.slice(2)),//To get arguments
    util = require('gulp-util');

const gulpConfig = require('./gulp-config.json');
const themes = gulpConfig.themes;
const watchThemes = gulpConfig.watch;

const defaultThemeName = themes[0],
    themeName = argv.theme ? argv.theme : defaultThemeName,
    production = Boolean(argv.production),
    stage = Boolean(argv.stage),
    dev = !production && !stage;

let watchProcess = false;

const paths = (function () {
    let getThemePath = function (theme = themeName) {
        return './www/wp-content/themes/' + theme;
    };

    let themeSrc = function (paths) {

        if (!Array.isArray(paths)) {
            paths = [paths];
        }

        return paths.map(function (path) {
            return getThemePath(path);
        })
    };

    let themeDest = function (path) {
        return getThemePath(path);
    };

    let themeAssetsPath = function (theme = themeName) {
        return getThemePath(theme) + '/assets';
    };

    let srcAsset = function (path) {
        return themeAssetsPath(defaultThemeName) + path;
    };

    return {
        themeSrc: themeSrc,
        themeDest: themeDest,
        themeAssetsPath: themeAssetsPath,
        srcAsset: srcAsset
    };
}());

const gulpTasks = (function () {
    return {
        concatFiles(src, destination, concatName) {
            return gulp.src(src).pipe(concat(concatName)).pipe(gulp.dest(destination));
        },
        scssTask(src, destination) {
            return gulp.src(src)
                .pipe(sassLint())
                .pipe(sassLint.format())
                .pipe((stage || production) ? sassLint.failOnError() : through2(
                    {objectMode: true}))
                .pipe((dev || stage) ? sourcemaps.init() : through2(
                    {objectMode: true}))
                .pipe(dev ? sass().on('error', (err) => {
                    util.log(err.message);
                    //this.emit('end');
                }) : sass().on('error', util.log))
                .pipe((stage || production) ? minifyCss({
                    compatibility: 'ie9',
                    level: 2,
                }) : through2({objectMode: true}))
                .pipe((stage || production) ? rename({
                    suffix: '.min',
                }) : through2({objectMode: true}))
                .pipe((dev || stage) ? sourcemaps.write() : through2(
                    {objectMode: true}))
                .pipe(autoprefixer({
                    browsers: ['last 2 versions', '> 1%'],
                    cascade: false
                }))
                .pipe(pxtorem())
                .pipe(gulp.dest(destination));
        },
        jsTask(src, destination, concatName) {
            // Just esLint first time on watch
            if (!watchProcess) {
                this.eslintFile(src);
            }

            return browserify({
                entries: src,
                debug: (dev || stage),
            })
                .bundle()
                .on('error', util.log)
                .pipe(source(dev ? concatName + '.js' : concatName + '.min.js'))
                .pipe(buffer())
                .pipe((stage || production) ? uglify() : through2(
                    {objectMode: true}))
                .on('error', util.log)
                .pipe((dev || stage) ? sourcemaps.init({loadMaps: true}) : through2(
                    {objectMode: true}))
                .pipe((dev || stage) ? sourcemaps.write('.') : through2(
                    {objectMode: true}))
                .pipe(gulp.dest(destination));
        },
        eslintFile(file) {
            return gulp.src(file instanceof Array ? file.filter(src => !src.endsWith('.min.js')) : file)
                .pipe(esLint())
                .pipe(esLint.format())
                .pipe((stage || production)
                    ? esLint.failAfterError()
                    : through2({objectMode: true}));

        },
        moveFiles(src, dest) {
            return gulp.src(src).pipe(gulp.dest(dest));
        },
    };
}());

/*
 |--------------------------------------------------------------------------
 | Tasks
 |--------------------------------------------------------------------------
 */

let themeConfigs = themes.map(function (themeName) {
    return {
        name: themeName,
        config: require(paths.themeAssetsPath(themeName) + '/config.json'),
    }
});

// Theme tasks
themeConfigs.forEach(function (theme) {
    let themeName = theme.name,
        themeConfig = theme.config,

        /** @var Contains names of the tasks that will be generated */
        themeStyleTasks = [],
        themeCopyFilesTasks = [],
        themeTaskGroup = [],
        themeJsTasks = [];

    themeConfig.style = themeConfig.style || [];
    themeConfig.js = themeConfig.js || [];
    themeConfig.copyFiles = themeConfig.copyFiles || [];

    // SCSS Compilation
    themeConfig.style.forEach(function (taskConfig, index) {
        let taskName = 'style:' + themeName + ':' + index;

        gulp.task(taskName, () => {
            return gulpTasks.scssTask(
                paths.themeSrc(taskConfig.src),
                paths.themeDest(taskConfig.dest)
            )
        })
        themeStyleTasks.push(taskName)
    })

    // Moving of files
    themeConfig.copyFiles.forEach(function (taskConfig, index) {
        let taskName = 'copy:' + themeName + ':' + index;

        gulp.task(taskName, () => {
            return gulpTasks.moveFiles(
                paths.themeSrc(taskConfig.src),
                paths.themeDest(taskConfig.dest)
            )
        })
        themeCopyFilesTasks.push(taskName)
    })

    // JS Compilation
    themeConfig.js.forEach(function (taskConfig, index) {
        let taskName = 'js:' + themeName + ':' + index;

        gulp.task(taskName, () => {
            return gulpTasks.jsTask(
                paths.themeSrc(taskConfig.src),
                paths.themeDest(taskConfig.dest),
                taskConfig.concatName,
                taskConfig.ingoreLint
            )
        })
        themeJsTasks.push(taskName)
    })

    // Setup one task for all theme related Scss tasks under "style:[themeName]"
    themeTaskGroup.push('style:' + themeName);
    gulp.task('style:' + themeName, themeStyleTasks);

    // Setup one task for all theme related Copy tasks under "copy:[themeName]"
    themeTaskGroup.push('copy:' + themeName);
    gulp.task('copy:' + themeName, themeCopyFilesTasks);

    // Setup one task for all theme related Js tasks under "js:[themeName]"
    themeTaskGroup.push('js:' + themeName);
    gulp.task('js:' + themeName, themeJsTasks);

    // Setup one theme task under "theme:[themeName]"
    gulp.task('theme:' + themeName, themeTaskGroup);
});

/*
* =======================================================
* Global Tasks that will effect alla themes:
* For running task for single theme run: [TASK]:[THEME_NAME]
* ex. theme:tryout
* =======================================================
*/

gulp.task('style', themes.map(theme => 'style:' + theme));
gulp.task('copy', themes.map(theme => 'copy:' + theme));
gulp.task('js', themes.map(theme => 'js:' + theme));
gulp.task('admin', themes.map(theme => 'admin:' + theme));
gulp.task('default', themes.map(theme => 'theme:' + theme));

/** ======================================================= */

/*
 |--------------------------------------------------------------------------
 | Watcher for development Not updated to support new structure New structure
 |--------------------------------------------------------------------------
 */
gulp.task('watch', ['default'], () => {
    let themeName;
    watchProcess = true;

    const watch = {
        style: [],
        js: [],
        onChange: (event) => {
            log('File ' + event.path + ' was ' + event.type +
                ', running tasks...');
        },
    };

    for (themeName in watchThemes) {
        let config = watchThemes[themeName]

        if (Object.hasOwnProperty.call(config, 'style')) {
            watch.style.push(paths.themeAssetsPath('/' + themeName) + config.style)
        }

        if (Object.hasOwnProperty.call(config, 'js')) {
            watch.js.push(paths.themeAssetsPath('/' + themeName) + config.js)
        }
    }


    gulp.watch(watch.style, ['style']).on('change', watch.onChange);
    gulp.watch(watch.js, ['js']).on('change', (event) => {
        log('File ' + event.path + ' was ' + event.type + ', running tasks...');

        gulpTasks.eslintFile(event.path);
    });
});
